const { readFileSync } = require('fs');
const path = require('path');
const requireResolve = require('require-resolve');

const {
  cssToJss,
  postcss,
} = require('./helpers');

/**
 * Visitor for `import '*.css'` babel AST-nodes
 */
function CssImport(cb) {
  return ({ node, replaceWith }, { file, opts = {} }) => {
    if (!node.source.value.endsWith('.css')) return;

    const { src } = requireResolve(node.source.value, path.resolve(file.opts.filename));
    const cssContent = readFileSync(src).toString();

    const { code, classes } = postcss.process(cssContent, src, opts);
    const jssObject = cssToJss({ code });

    cb({
      src,
      css: cssContent,
      replaceWith,
      importNode: { ...node, ...node.specifiers[0] },
      jssObject,
      classes,
    });
  };
}

module.exports = CssImport;
